/*
** Copyright (C) 2009 Patrick Kirchhof, Kevin Suske and Pasqual Dhring
** 
** This file is part of GFA.
** 
** GFA is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** GFA is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with GFA.  If not, see <http://www.gnu.org/licenses/>.
** 
*/

package animator.src;

import java.util.Vector;

import reporter.interfaces.IReporterFacade;
import animator.src.AnimatorXML;

import org.w3c.dom.Element;

public class AnimatorFacade implements IReporterFacade {
	
	// used to keep track of all transporters
	private Vector<String> transporters = new Vector<String>();
	private String xmlName;
	private AnimatorXML axml;
	private Element trace, struct;
	
	public AnimatorFacade() {
		axml = new AnimatorXML(true, true);
		struct = axml.createNodeStructure();
		trace = axml.createNodeTrace();		
	}

	@Override
	public void addConveyor(String uuid, String name,
			String originUUID, String destinationUUID, double stdtraveltime) {
		struct.appendChild(axml.createNodeConveyor(uuid, name, originUUID, destinationUUID, stdtraveltime, 0, 0, 0, 0));
	}
	
	@Override
	public void addConveyorState(String state) {	
	}

	@Override
	public void addCustomer(String uuid, String name) {
	}

	@Override
	public void addItem(String uuid, String name) {
		struct.appendChild(axml.createNodeItem(uuid, name, "", ""));	
	}		
	
	@Override
	public void addJob(String uuid, String customerReferenceUUID,
			double arrivaltime, String itemUuid, double posx, double posy) {	
		trace.appendChild(axml.createNodeJob(uuid, customerReferenceUUID, arrivaltime, itemUuid, posx, posy));
	}

	@Override
	public void addJobState(String state) {	
	}
	
	@Override
	public void addNode(String uuid, String name, String type, double posx,
			double posy) {
		axml.addNode(uuid, type, posx, posy);
		
		if (type.equals("sink")) {
			struct.appendChild(axml.createNodeSink(uuid, name, type, posx, posy, "", "0"));	
		}
		
		if (type.equals("source")) {
			struct.appendChild(axml.createNodeSource(uuid, name, type, posx, posy, "", "0"));
		}
		
		if (type.equals("parking")) {
			struct.appendChild(axml.createNodeParking(uuid, name, type, posx, posy));
		}
	}

	@Override
	public void addPath(String uuid, String name, String originUUID,
			String destinationUUID, boolean isBidirectional, double stdtraveltime) {	
		struct.appendChild(axml.createNodePath(uuid, name, originUUID, destinationUUID, stdtraveltime, 0, 0, 0, 0));
	}	

	@Override
	public void addPersonnelState(String state) {
	}

	@Override
	public void addQueue (String uuid, String name, String description, 
			double posxIn, double posyIn, double posxOut, double posyOut) {
		struct.appendChild(axml.createNodeQueue(uuid, name, posxIn, posyIn, posxOut, posyOut, "", ""));
	}

	@Override
	public void addResource(String uuid, String name, String description,
			double capacity, double posx, double posy, String type) {
		if (type.equals("transporter")) {
			transporters.add(uuid);
			struct.appendChild(axml.createNodeTransporter(uuid, name, posx, posy, type, ""));
		}
		
		if (type.equals("workstation")) {
			struct.appendChild(axml.createNodeWorkstation(uuid, name, posx, posy, type, "", "0"));
		}
	}
	

	@Override
	public void addServerState(String state) {
	}

	@Override
	public void addShopFloor(String uuid, String name, String description) {
	}

	@Override
	public void addSingleValueVariable(String uuid, String name,
			String description, double value) {
	}

	@Override
	public void addSink(String uuid, String name, double posx, double posy) {
	}

	@Override
	public void addSource(String uuid, String name, double posx, double posy) {	
	}	
	
	@Override
	public void addTimeDependent(String uuid, String name, String description) {	
	}

	@Override
	public void addTimeDependentValue(String uuid, double time, double value) {	
	}

	@Override
	public void addTimeIndependent(String uuid,String name, String description) {
	}

	@Override
	public void addTimeIndependentValue(String uuid, double value) {
	}

	@Override
	public void addTransporterState(String state) {
	}

	@Override
	public void addWarehouse(String uuid, String name, String description) {
	}

	@Override
	public void registerExperiment(String title, String analyst,
			String description) {
		xmlName = title;
	}

	@Override
	public void reportArrival(String uuid, String queueReferenceUUID,
			double time) {
		trace.appendChild(axml.createNodeInsert(uuid, queueReferenceUUID, time));
	}

	@Override
	public void reportDispatch(String uuid, String queueReferenceUUID, String resourceReferenceUUID,
			double crtTime, double serviceTime) {
		// Dispatch bedeutet hier, dass ein Auftrag an einer Warteschlange entnommen und eingelastet wurde, dh
		// die Bearbeitung wurde begonnen. uuid bezieht sich auf den Job. queueReferenceUUID ist die uuid der 
		// Warteschlange. serviceTime ist die tatschliche BEarbeitungszeit des Jobs.
		trace.appendChild(axml.createNodeRemove(uuid, queueReferenceUUID, resourceReferenceUUID, crtTime, serviceTime));
		trace.appendChild(axml.createNodeSeize(uuid, queueReferenceUUID, resourceReferenceUUID, crtTime, serviceTime));
	}
	
	@Override
	public void reportJobPositionChange(String uuid, double time, double posX,
			double posY) {
		trace.appendChild(axml.createNodeMoveJob(uuid, time, posX, posY));
	}

	@Override
	public void reportJobStatusChange(String uuid, double time, String newStatus, double posx, double posy) {
		// Status types: released, queued, processingRequested, processingStarted, processingFinished,
		// transportRequested, transportFinished, completed, delivered		
		if (newStatus.equals("released")) {
			trace.appendChild(axml.createNodeCreate(uuid, time, newStatus, posx, posy));
		}
		
		if (newStatus.equals("delivered")) {
			trace.appendChild(axml.createNodeTerminate(uuid, time, newStatus, posx, posy));
		}
	}
	
	@Override
	public void reportLoaded(String uuid, String transporterUuid, double time,
			double posX, double posY) {
		trace.appendChild(axml.createNodeLoadTransporter(uuid, transporterUuid, time, posX, posY));
	}

	@Override
	public void reportQLenght(String uuid, double time, double lenght) {
	}
	
	@Override
	public void reportResourcePosition(String uuid, double time, double posx,
			double posy) {
		for (int i = 0; i < transporters.size(); i++) {
			// check if resource is a transporter
			if (transporters.get(i).equals(uuid)) {
				trace.appendChild(axml.createNodeMoveTransporter(uuid, time, posx, posy));
			}
		}
	}	

	@Override
	public void reportResourceStatusChange(String uuid, double time, String status) {
		// WICHTIG: Hier gibt es folgende Status: idle, busy, breakdown, stopped, maintained, setup
		// bei jedem Ereignis, das eine Vernderung verursachen kann, wird diese Methode aufgerufen, 
		// auch wenn sich der Status tatschlich nicht ndert!
		trace.appendChild(axml.createNodeChangeState(uuid, time, status));
	}

	@Override
	public void reportResourceUsage(String uuid, double time, double usedCapacity) {
	}
	
	@Override
	public void reportSimulationEnd(double time) {
		axml.finishDocument(xmlName, struct, trace, true);
	}	
	
	@Override
	public void reportUnloaded(String uuid, String transporterUuid,
			double time, double posX, double posY) {
		trace.appendChild(axml.createNodeUnloadTransporter(uuid, transporterUuid, time, posX, posY));
	}

	@Override
	public void setLength(double simulationlength) {
	}

	@Override
	public void setTimeUnits(String timeunit) {
	}

	@Override
	public void setWarmupPeriod(double warmupperiod) {	
	}
}
