/*
** Copyright (C) 2009 Patrick Kirchhof, Kevin Suske and Pasqual Dhring
** 
** This file is part of GFA.
** 
** GFA is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** GFA is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with GFA.  If not, see <http://www.gnu.org/licenses/>.
** 
*/

package animator.src;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.w3c.dom.Element;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

public class AnimatorFrontend extends JFrame {		
	
	private static final long serialVersionUID = 7210859880737742154L;
	private JFrame frontendWindow;
	private JDialog optionsWindow;
	private JButton bexit, babout, boptions, bgenerate, bok, bclear, bsavexml;
	private JCheckBox cbgrid, cblabels;
	private JComboBox cbworkstation, cbsource, cbsink, cbtransporter, cbjob;
	private JLabel lduration, ljob, lsink, lsource, ltransporter, lworkstation;
	private JPanel filePane, optionsPane, options1Pane, buttonPane, button2Pane, 
		jbChooserPane, siChooserPane, soChooserPane, tpChooserPane, wsChooserPane;
	private JSpinner sduration;
	private AnimatorFileBrowser structPane, tracePane, jobPane, transporterPane, sourcePane, sinkPane, workstationPane;
	public static boolean xmlLocationChanged = true;
	
	/**
	 * Run frontend
	 */
	public static void main(String[] args) {
		AnimatorFrontend af = new AnimatorFrontend();	
		af.buildGUI();
	}
	
	/**
	 * Build the frontend GUI
	 */
	private void buildGUI() {	
		createFrontendComponents();
		configureFrontendMouseListeners();
		configureFrontendComponents();
		
		frontendWindow.setResizable(false);
		frontendWindow.setTitle("Generic Factory Animator");
		frontendWindow.setSize(450, 150);
		
		frontendWindow.setUndecorated(true);
		frontendWindow.getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);
		
		frontendWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	
		frontendWindow.setLocationRelativeTo(null);	

		frontendWindow.pack();
		frontendWindow.setVisible(true);	
	}
	
	/**
	 * Build the options GUI
	 */
	private void buildOptionsGUI() {			
		createOptionsComponents();
		configureOptionsMouseListeners();
		configureOptionsItemListeners();
		configureOptionsComponents();
		
		optionsWindow.setResizable(false);
		optionsWindow.setTitle("Template Configuration");
		optionsWindow.setSize(450, 475);
		
		optionsWindow.setUndecorated(true);
		optionsWindow.getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);
		
		optionsWindow.setModal(true);
		optionsWindow.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		optionsWindow.setLocationRelativeTo(frontendWindow);
		
		optionsWindow.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
	    	  	closeOptionsGUI();
			}
		});
		
		optionsWindow.setVisible(true);		
	}
	
	/**
	 * save currently selected values and dispose options window
	 */
	private void closeOptionsGUI() {  	
		saveOptionsValues("", "");    	
    	optionsWindow.setVisible(false);
    	optionsWindow.dispose();
	}
	
	/**
	 * Create the components needed for the frontend
	 */
	private void createFrontendComponents() {
		// AnimatorFileBrowser
		structPane = new AnimatorFileBrowser("Structure", 10, "xml", false, false);
		tracePane = new AnimatorFileBrowser("Trace", 30, "xml", false, false);
		// Button
		bexit = new JButton("Exit");
		babout = new JButton("About");
		boptions = new JButton("Templates");
		bgenerate = new JButton("Generate X3D");
		// Checkbox
		cbgrid = new JCheckBox("Disable Grid");
		cblabels = new JCheckBox("Disable Labels");
		// Frame
		frontendWindow = new JFrame();			
		// Label
		lduration = new JLabel("Animation Length (sec)");
		// Panel
		filePane = new JPanel();
		optionsPane = new JPanel();
		options1Pane = new JPanel();
		buttonPane = new JPanel();
		// Spinner
		sduration = new JSpinner(new SpinnerNumberModel(AnimatorUtils.animationLength, 10, 9000, 10));
		sduration.setPreferredSize(new Dimension(50, 20));
	}
	
	private void createOptionsComponents() {
		// AnimatorFileBrowser
		jobPane = new AnimatorFileBrowser("", 105, "x3d", true, false);
		sinkPane = new AnimatorFileBrowser("", 105, "x3d", false, true);
		sourcePane = new AnimatorFileBrowser("", 105, "x3d", false, true);
		transporterPane = new AnimatorFileBrowser("", 105, "x3d", false, false);
		workstationPane = new AnimatorFileBrowser("", 105, "x3d", false, true);
		// Button
		bclear = new JButton("Clear");
		bsavexml = new JButton("Save to XML");
    	bok = new JButton("OK");
    	// Combobox
    	cbjob = new JComboBox();
    	cbsink = new JComboBox();
    	cbsource = new JComboBox();
    	cbtransporter = new JComboBox();
    	cbworkstation = new JComboBox();
		// Frame
		optionsWindow = new JDialog();
		// Label
		ljob = new JLabel("Items");
		lsink = new JLabel("Sinks");
		lsource = new JLabel("Sources");
		ltransporter = new JLabel("Transporter");
    	lworkstation = new JLabel("Workstations");
    	// Panel
    	button2Pane = new JPanel(); 
    	jbChooserPane = new JPanel();
    	siChooserPane = new JPanel();
    	soChooserPane = new JPanel();
    	tpChooserPane = new JPanel();
    	wsChooserPane = new JPanel();
	}
	
	/**
	 * Configure and place the components on the content pane
	 */
	private void configureFrontendComponents() {	
		structPane.setText(AnimatorUtils.defaultStructureFile);
		tracePane.setText(AnimatorUtils.defaultTraceFile);	
		
		filePane.setLayout(new BoxLayout(filePane, BoxLayout.Y_AXIS));
		filePane.add(structPane);
		filePane.add(tracePane);
		
		optionsPane.setLayout(new BoxLayout(optionsPane, BoxLayout.Y_AXIS));
		optionsPane.add(options1Pane);
		
		options1Pane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		options1Pane.setLayout(new BoxLayout(options1Pane, BoxLayout.LINE_AXIS));
		options1Pane.add(lduration); options1Pane.add(Box.createRigidArea(new Dimension(15, 0)));
		options1Pane.add(sduration); options1Pane.add(Box.createRigidArea(new Dimension(20, 0)));
		options1Pane.add(cbgrid); options1Pane.add(Box.createRigidArea(new Dimension(10, 0)));
		options1Pane.add(cblabels);
		
		buttonPane.add(babout); buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(Box.createHorizontalGlue());
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));
		buttonPane.add(bgenerate); buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(boptions); buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(bexit);
		buttonPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));		
		
		Container contentPane = frontendWindow.getContentPane();
		contentPane.add(filePane, BorderLayout.PAGE_START);
		contentPane.add(optionsPane, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);
	}
	
	/**
	 * Configure and place the components on the content pane
	 */
	private void configureOptionsComponents() {
		Container contentPane = optionsWindow.getContentPane();
		contentPane.setLayout(new GridLayout(0,1));
		populateOptionsChoosers();
		
		jbChooserPane.setLayout(new BoxLayout(jbChooserPane, BoxLayout.LINE_AXIS));
		jbChooserPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,12));
		jbChooserPane.add(ljob); jbChooserPane.add(Box.createRigidArea(new Dimension(73, 0)));
		jbChooserPane.add(cbjob);
		
		siChooserPane.setLayout(new BoxLayout(siChooserPane, BoxLayout.LINE_AXIS));
		siChooserPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,11));
		siChooserPane.add(lsink); siChooserPane.add(Box.createRigidArea(new Dimension(73, 0)));
		siChooserPane.add(cbsink);
		
		soChooserPane.setLayout(new BoxLayout(soChooserPane, BoxLayout.LINE_AXIS));
		soChooserPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,11));
		soChooserPane.add(lsource); soChooserPane.add(Box.createRigidArea(new Dimension(57, 0)));
		soChooserPane.add(cbsource);
		
		tpChooserPane.setLayout(new BoxLayout(tpChooserPane, BoxLayout.LINE_AXIS));
		tpChooserPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,11));
		tpChooserPane.add(ltransporter); tpChooserPane.add(Box.createRigidArea(new Dimension(37, 0)));
		tpChooserPane.add(cbtransporter);
		
		wsChooserPane.setLayout(new BoxLayout(wsChooserPane, BoxLayout.LINE_AXIS));
		wsChooserPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,11));
		wsChooserPane.add(lworkstation); wsChooserPane.add(Box.createRigidArea(new Dimension(26, 0)));
		wsChooserPane.add(cbworkstation);
		    	
		contentPane.add(jbChooserPane); contentPane.add(jobPane);
		contentPane.add(siChooserPane); contentPane.add(sinkPane);
		contentPane.add(soChooserPane); contentPane.add(sourcePane);
		contentPane.add(tpChooserPane); contentPane.add(transporterPane);
		contentPane.add(wsChooserPane); contentPane.add(workstationPane);
	    	  	
		button2Pane.add(Box.createHorizontalGlue());
		button2Pane.setLayout(new BoxLayout(button2Pane, BoxLayout.LINE_AXIS));
		button2Pane.add(bok); button2Pane.add(Box.createRigidArea(new Dimension(10, 0)));
		button2Pane.add(bsavexml); button2Pane.add(Box.createRigidArea(new Dimension(10, 0)));
		button2Pane.add(bclear); 
		button2Pane.setBorder(BorderFactory.createEmptyBorder(5, 10, 0, 10));
		contentPane.add(button2Pane);
	}
	
	/**
	 * Configure the mouse listeners for all "bottom-buttons" (frontend)
	 */
	private void configureFrontendMouseListeners() {
		// Exit
		bexit.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	System.exit(0);
            }
		});	
		// About
		babout.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	JOptionPane.showMessageDialog(frontendWindow, 
            			"GFA Copyright (C) 2009  Patrick Kirchhof, Kevin Suske and Pasqual Dhring\n\n" +
            			"This is free software, and you are welcome to redistribute it under certain conditions.\n" +
            			"This program comes with ABSOLUTELY NO WARRANTY.\n\n" +
            			"See the GNU General Public License for more details.", 
            			"About", JOptionPane.INFORMATION_MESSAGE);
            }
		});
		// Options
		boptions.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	if (structPane.getAbsolutePath().equals("")) {
            		JOptionPane.showMessageDialog(frontendWindow, "Select the source files first.", "Error", JOptionPane.ERROR_MESSAGE);
            	} else {
            		AnimatorUtils.parseMode = AnimatorUtils.OPTIONS;		            		
            		if (xmlLocationChanged) {
            			AnimatorBuilder.objects.clear();
            			AnimatorXML.parseXMLFile(structPane.getAbsolutePath(), tracePane.getAbsolutePath());		            	
                		xmlLocationChanged = false;
            		}    
	            	buildOptionsGUI();
            	}		            			            		            			    
            }
		});		
		// Generate
		bgenerate.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	if (structPane.getAbsolutePath().equals("")) {
        			JOptionPane.showMessageDialog(frontendWindow, "Select the source files first.", "Error", JOptionPane.ERROR_MESSAGE);
            	} else {
            		// if the template objects not initialized, initialize them now
            		if (AnimatorBuilder.objects.size() == 0) {
            			AnimatorUtils.parseMode = AnimatorUtils.OPTIONS;
            			AnimatorXML.parseXMLFile(structPane.getAbsolutePath(), tracePane.getAbsolutePath());
            		}
            		// set options and start parsing
            		AnimatorUtils.parseMode = AnimatorUtils.NONE;
            		AnimatorUtils.animationLength = (Integer)sduration.getValue();
            		AnimatorUtils.showGrid = !cbgrid.isSelected();
            		AnimatorUtils.enableLabels = !cblabels.isSelected();		            		
            		AnimatorXML.parseXMLFile(structPane.getAbsolutePath(), tracePane.getAbsolutePath());
            		JOptionPane.showMessageDialog(frontendWindow, "Conversion finished!", "Information", JOptionPane.INFORMATION_MESSAGE);
            	}
            }
		});
	}
	
	/**
	 * Configure the item listeners for all combo boxes
	 */
	private void configureOptionsItemListeners() {
		cbjob.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				if (optionsWindow.isVisible()) {
					if (e.getStateChange() == ItemEvent.DESELECTED) saveOptionsValues("job", (String)e.getItem());
					if (e.getStateChange() == ItemEvent.SELECTED) loadOptionsValues("job");
				}				          
            }
        });
		cbsink.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				if (optionsWindow.isVisible()) {
					if (e.getStateChange() == ItemEvent.DESELECTED) saveOptionsValues("sink", (String)e.getItem());
					if (e.getStateChange() == ItemEvent.SELECTED) loadOptionsValues("sink");
				}				          
            }
        });
		cbsource.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				if (optionsWindow.isVisible()) {
					if (e.getStateChange() == ItemEvent.DESELECTED) saveOptionsValues("source", (String)e.getItem());
					if (e.getStateChange() == ItemEvent.SELECTED) loadOptionsValues("source");
				}				          
            }
        });
		cbtransporter.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				if (optionsWindow.isVisible()) {
					if (e.getStateChange() == ItemEvent.DESELECTED) saveOptionsValues("transporter", (String)e.getItem());
					if (e.getStateChange() == ItemEvent.SELECTED) loadOptionsValues("transporter");
				}				          
            }
        });
		cbworkstation.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				if (optionsWindow.isVisible()) {
					if (e.getStateChange() == ItemEvent.DESELECTED) saveOptionsValues("workstation", (String)e.getItem());
					if (e.getStateChange() == ItemEvent.SELECTED) loadOptionsValues("workstation");
				}				          
            }
        });
	}
	
	/**
	 * Configure the mouse listeners for all "bottom-buttons" (options)
	 */
	private void configureOptionsMouseListeners() {
		// OK
		bok.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	closeOptionsGUI();
            }
		});
		// Save to XML
		bsavexml.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	saveOptionsToXML();
            	JOptionPane.showMessageDialog(optionsWindow, "Configuration saved!", "Information", JOptionPane.INFORMATION_MESSAGE);
            }
		});
		// Clear
		bclear.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
            	// clear gui
            	sinkPane.setRotation("0"); sourcePane.setRotation("0"); workstationPane.setRotation("0");
            	jobPane.setColor(null); jobPane.clearText(); transporterPane.clearText();
            	sourcePane.clearText(); sinkPane.clearText(); workstationPane.clearText();
            	// clear objects
            	Iterator it = AnimatorBuilder.objects.entrySet().iterator();
            	while (it.hasNext()) { 
            		Map.Entry entry = (Map.Entry) it.next();
            		AnimatorObject ao = (AnimatorObject)(entry).getValue();
            		ao.setTemplateURL("");
            		ao.setColor(null);
            		ao.setRotation("0");
            	}		            	
            }
		});
	}
	
	/**
	 * Load the value for the currently selected item into the GUI
	 */
	private void loadOptionsValues(String type) {
		Iterator it = AnimatorBuilder.objects.entrySet().iterator();
    	while (it.hasNext()) { 
    		Map.Entry entry = (Map.Entry) it.next();
    		AnimatorObject ao = (AnimatorObject)(entry).getValue();
    		// load items
    		if (type.equals("job") && ao.getType().equals("item") && ao.getName().equals(cbjob.getSelectedItem())) {
    			jobPane.setText(ao.getTemplateURL());
    			jobPane.setColor(ao.getColor());
    		}    
    		// load sinks
    		if (type.equals("sink") && ao.getType().equals("sink") && ao.getName().equals(cbsink.getSelectedItem())) {
    			sinkPane.setText(ao.getTemplateURL());
    			sinkPane.setRotation(ao.getRotation()+"");
    		}
    		// load sources
    		if (type.equals("source") && ao.getType().equals("source") && ao.getName().equals(cbsource.getSelectedItem())) {
    			sourcePane.setText(ao.getTemplateURL());
    			sourcePane.setRotation(ao.getRotation()+"");
    		}    
    		// load transporters
    		if (type.equals("transporter") && ao.getType().equals("transporter") && ao.getName().equals(cbtransporter.getSelectedItem())) {
    			transporterPane.setText(ao.getTemplateURL());
    		}   
    		// load workstations
    		if (type.equals("workstation") && ao.getType().equals("workstation") && ao.getName().equals(cbworkstation.getSelectedItem())) {
    			workstationPane.setText(ao.getTemplateURL());
    			workstationPane.setRotation(ao.getRotation()+"");
    		}    			
    	}
	}
	
	/**
	 * Populate the combo boxes with the model components
	 */
	private void populateOptionsChoosers() {
		Vector<String> vim = new Vector<String>(); Vector<String> vsi = new Vector<String>(); 
		Vector<String> vso = new Vector<String>(); Vector<String> vtp = new Vector<String>(); 
		Vector<String> vws = new Vector<String>();
		ComboBoxModel model;
		// get objects from model
		Iterator it = AnimatorBuilder.objects.entrySet().iterator();
    	while (it.hasNext()) { 
    		Map.Entry entry = (Map.Entry) it.next();
    		AnimatorObject ao = (AnimatorObject)(entry).getValue();
    		if (ao.getType().equals("item")) vim.add(ao.getName());
    		if (ao.getType().equals("sink")) vsi.add(ao.getName());
    		if (ao.getType().equals("source")) vso.add(ao.getName());
    		if (ao.getType().equals("transporter")) vtp.add(ao.getName());
    		if (ao.getType().equals("workstation")) vws.add(ao.getName());
    	}
    	// sorting
    	Collections.sort(vim); model = new DefaultComboBoxModel(vim); cbjob.setModel(model);
    	Collections.sort(vsi); model = new DefaultComboBoxModel(vsi); cbsink.setModel(model);
    	Collections.sort(vso); model = new DefaultComboBoxModel(vso); cbsource.setModel(model);
    	Collections.sort(vtp); model = new DefaultComboBoxModel(vtp); cbtransporter.setModel(model);
    	Collections.sort(vws); model = new DefaultComboBoxModel(vws); cbworkstation.setModel(model);
    	// load values for selected items
    	loadOptionsValues("job");
    	loadOptionsValues("sink");
    	loadOptionsValues("source");
    	loadOptionsValues("transporter");
    	loadOptionsValues("workstation");
	}
	
	/**
	 * Save the template configuration to the xml structure file
	 */
	private void saveOptionsToXML() {
		saveOptionsValues("", "");  
		
		AnimatorXML axml = new AnimatorXML(true, false);
		Element struct = axml.createNodeStructure();
		
		Iterator it = AnimatorBuilder.objects.entrySet().iterator();
    	while (it.hasNext()) { 
    		Map.Entry entry = (Map.Entry) it.next();
    		AnimatorObject ao = (AnimatorObject)(entry).getValue();
    		
    		if (ao.getType().equals("conveyor")) {
    			struct.appendChild(axml.createNodeConveyor(ao.getUUID(), ao.getName(), "", "", Double.valueOf(ao.getStdTravelTime()), 
    					ao.getPositionX(true), ao.getPositionY(true), ao.getPositionX(false), ao.getPositionY(false)));
    		}
    		if (ao.getType().equals("parking")) {
    			struct.appendChild(axml.createNodeParking(ao.getUUID(), ao.getName(), ao.getType(), ao.getPositionX(false), ao.getPositionY(false)));
    		}
    		if (ao.getType().equals("path")) {
    			struct.appendChild(axml.createNodePath(ao.getUUID(), ao.getName(), "", "", Double.valueOf(ao.getStdTravelTime()),
    					ao.getPositionX(true), ao.getPositionY(true), ao.getPositionX(false), ao.getPositionY(false)));
    		}
    		if (ao.getType().equals("queue")) {
    			struct.appendChild(axml.createNodeQueue(ao.getUUID(), ao.getName(), ao.getPositionX(true), ao.getPositionY(true), ao.getPositionX(false), ao.getPositionY(false),
    					"", ""));
    		}
    		if (ao.getType().equals("item")) {
    			struct.appendChild(axml.createNodeItem(ao.getUUID(), ao.getName(), ao.getTemplateURL(), ao.getColorXML()));
    		}
    		if (ao.getType().equals("sink")) {
    			struct.appendChild(axml.createNodeSink(ao.getUUID(), ao.getName(), ao.getType(), ao.getPositionX(false), ao.getPositionY(false), ao.getTemplateURL(), ao.getRotation()));
    		}
    		if (ao.getType().equals("source")) {
    			struct.appendChild(axml.createNodeSource(ao.getUUID(), ao.getName(), ao.getType(), ao.getPositionX(false), ao.getPositionY(false), ao.getTemplateURL(), ao.getRotation()));
    		}
    		if (ao.getType().equals("transporter")) {
    			struct.appendChild(axml.createNodeTransporter(ao.getUUID(), ao.getName(), ao.getPositionX(false), ao.getPositionY(false), ao.getType(), ao.getTemplateURL()));
    		}
    		if (ao.getType().equals("workstation")) {
    			struct.appendChild(axml.createNodeWorkstation(ao.getUUID(), ao.getName(), ao.getPositionX(false), ao.getPositionY(false), ao.getType(), ao.getTemplateURL(), ao.getRotation()));
    		}
    	}

    	axml.finishDocument(structPane.getAbsolutePath(), struct, null, false);
	}
	
	/**
	 * Save the deselected value
	 */
	private void saveOptionsValues(String type, String deselected) {
		boolean saveAll = false; if (type.isEmpty()) saveAll = true; 
		Iterator it = AnimatorBuilder.objects.entrySet().iterator();
    	while (it.hasNext()) { 
    		Map.Entry entry = (Map.Entry) it.next();
    		AnimatorObject ao = (AnimatorObject)(entry).getValue();
    		// save items
    		if (saveAll) { type = "job"; deselected = (String)cbjob.getSelectedItem(); }
    		if (type.equals("job") && ao.getType().equals("item") && ao.getName().equals(deselected)) {    			
    			ao.setTemplateURL(jobPane.getAbsolutePath());
    			ao.setColor(jobPane.getColor());
    		}
    		// save sinks
    		if (saveAll) { type = "sink"; deselected = (String)cbsink.getSelectedItem(); }
    		if (type.equals("sink") && ao.getType().equals("sink") && ao.getName().equals(deselected)) {
    			ao.setTemplateURL(sinkPane.getAbsolutePath());
    			ao.setRotation(sinkPane.getRotation());
    		}		
    		// save sources
    		if (saveAll) { type = "source"; deselected = (String)cbsource.getSelectedItem(); }
    		if (type.equals("source") && ao.getType().equals("source") && ao.getName().equals(deselected)) {
    			ao.setTemplateURL(sourcePane.getAbsolutePath());
    			ao.setRotation(sourcePane.getRotation());
    		}
    		// save transporters
    		if (saveAll) { type = "transporter"; deselected = (String)cbtransporter.getSelectedItem(); }
    		if (type.equals("transporter") && ao.getType().equals("transporter") && ao.getName().equals(deselected)) {
    			ao.setTemplateURL(transporterPane.getAbsolutePath());
    		}   
    		// save workstations
    		if (saveAll) { type = "workstation"; deselected = (String)cbworkstation.getSelectedItem(); }
    		if (type.equals("workstation") && ao.getType().equals("workstation") && ao.getName().equals(deselected)) {
    			ao.setTemplateURL(workstationPane.getAbsolutePath());
    			ao.setRotation(workstationPane.getRotation());
    		}
    	}
	}
}