/*
** Copyright (C) 2009 Patrick Kirchhof, Kevin Suske and Pasqual Dhring
** 
** This file is part of GFA.
** 
** GFA is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** GFA is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with GFA.  If not, see <http://www.gnu.org/licenses/>.
** 
*/

package animator.src;

import java.awt.Color;

import java.util.Vector;

public class AnimatorObject {
	private String defID, name, type, uuid;
	private String stdTravelTime = "", templateURL = "", transporterUUID = "";
	private String x1, y1, z1, x2, y2, z2;						// entry and exit positions of object
	private Vector<String> eventTimes = new Vector<String>();	// time of event
	private Vector<String> eventStatus = new Vector<String>(); 	// uuid of destination
	private Vector<String> eventPosition = new Vector<String>();// position of destination
	private Vector<String> eventUUID = new Vector<String>();	// uuid of destination
	private int eventCount = 0;									// number of events in event vectors
	private boolean transported = false;						// true if object is currently being transported
	private Color color;
	private String rotation = "0";
	
	/**
	 * Default constructor
	 */
	public AnimatorObject(String defID, String uuid, String name, String type) {
		this.defID = defID; this.uuid = uuid; this.name = name; this.type = type;
	}
	
	/**
	 * Constructor for workstations
	 *
	public AnimatorObject(String defID, String uuid, String name, String type, String nodeUUID) {
		this.defID = defID; this.uuid = uuid; this.name = name; this.type = type; this.nodeUUID = nodeUUID;
	}
	
	/**
	 * Constructor for paths
	 *
	public AnimatorObject(String defID, String uuid, String name, String type, String entryUUID, String exitUUID) {
		this.defID = defID; this.uuid = uuid; this.name = name; this.type = type; this.entryUUID = entryUUID; this.exitUUID = exitUUID;
	}*/
	
	public void addEvent(String time, String status) {
		eventTimes.add(eventCount, time); eventStatus.add(eventCount, status);
		eventCount = eventCount+1;
	}
	
	public void addEvent(String time, String status, String position, String eUUID) {
		eventTimes.add(eventCount, time); eventStatus.add(eventCount, status); 
		eventPosition.add(eventCount, position); eventUUID.add(eventCount, eUUID);
		eventCount = eventCount+1;
	}
	
	public void clearEvents() {
		eventTimes.clear();
		eventStatus.clear();
		eventPosition.clear();
		eventUUID.clear();
		eventCount = 0;
	}
	
	public Color getColor() {
		return color;
	}
	
	public String getColorX3D() {
		if (color == null) return "";
		else {
			double r = (double)color.getRed()/255;
			double g = (double)color.getGreen()/255;
			double b = (double)color.getBlue()/255;
			return r + " " + g + " " + b;			
		}
	}
	
	public String getColorXML() {
		if (color == null) return "";
		else {
			double r = (double)color.getRed();
			double g = (double)color.getGreen();
			double b = (double)color.getBlue();
			return r + " " + g + " " + b;			
		}
	}
	
	public String getDefID() { 
		return defID;
	}
	
	public int getEventCount() {
		return eventCount;
	}
	
	public String getEventPosition(int number) {
		return eventPosition.get(number);
	}
	
	public String getEventStatus(int number) {
		return eventStatus.get(number);
	}
	
	public String getEventTime(int number) {
		return eventTimes.get(number);
	}
	
	public String getEventUUID(int number) {
		return eventUUID.get(number);
	}
	
	public String getLength() {
		double d = AnimatorUtils.getDistance(Double.parseDouble(x1), Double.parseDouble(x2), Double.parseDouble(y1), Double.parseDouble(y2));
		return Double.toString(d);
	}
	
	public String getName() {
		return name;
	}
	
	public String getPosition(boolean entryPoint, boolean resetZ, boolean x3dFormat) {
		String pos, x, y, z;
		if (entryPoint) { x = x1; y = y1; z = z1; }
			else { x = x2; y = y2; z = z2; }
		if (resetZ) z = "0";
		if (x3dFormat) pos = x+" "+z+" -"+y;
			else pos = x+" "+y+" "+z;
		return pos;
	}
	
	public double getPositionX(boolean entryPoint) {
		if (entryPoint) return Double.valueOf(x1);
		else return Double.valueOf(x2);
	}
	
	public double getPositionY(boolean entryPoint) {
		if (entryPoint) return Double.valueOf(y1);
		else return Double.valueOf(y2);
	}
	
	public String getRotation() {
		return rotation;
	}
	
	public String getRotationX3D() {
		double radians = Math.toRadians(Double.parseDouble(rotation));
		return "0 1 0 " + String.valueOf(radians);
	}
	
	public String getStdTravelTime() {
		return stdTravelTime;
	}
	
	public String getTemplateURL() {
		if (templateURL == null) return ""; else return templateURL;
	}
	
	public boolean getTransported() {
		return transported;
	}
	
	public String getTransporterUUID() {
		return transporterUUID;
	}
	
	public String getType() {
		return type;
	}
	
	public String getUUID() {
		return uuid;
	}
	
	public void resetObject() {
		clearEvents();
		transported = false;
		x1 = ""; y1 = ""; z1 = ""; x2 = ""; y2 = ""; z2 = "";
		stdTravelTime = ""; transporterUUID = "";
	}
	
	public void setColor(Color color) {
		this.color = color;
	}
	
	public void setPosition(String x, String y, String z) {
		setPosition(x, y, z, x, y, z);
	}
	
	public void setPosition(String x1, String y1, String z1, String x2, String y2, String z2) {
		this.x1 = x1; this.y1 = y1; this.z1 = z1; this.x2 = x2; this.y2 = y2; this.z2 = z2;
	}
	
	public void setPositionByString(String pos, boolean x3dFormat) {
		x1 = pos.substring(0, pos.indexOf(" "));
		pos =pos.substring(pos.indexOf(" ")+1, pos.length());
		if (x3dFormat) {
			z1 = pos.substring(0, pos.indexOf(" ")); 
			y1 = pos.substring(pos.indexOf("-")+1, pos.length());
		} else {
			y1 = pos.substring(0, pos.indexOf(" "));
			z1 = pos.substring(pos.indexOf(" ")+1, pos.length());
		}
		x2 = x1; y2 = y1; z2 = z1;
	}
	
	public void setRotation(String rotation) {
		if (rotation.indexOf("") == -1) this.rotation = rotation;
		else this.rotation = rotation.substring(0, rotation.indexOf(""));
	}
	
	public void setStdTravelTime(String stdTravelTime) {
		this.stdTravelTime = stdTravelTime;
	}
	
	public void setTemplateURL(String templateURL) {
		this.templateURL = templateURL;
	}
	
	public void setTransported(boolean transported, String transporterUUID) {
		if (AnimatorUtils.enableTransporters) {
			this.transported = transported; this.transporterUUID = transporterUUID;
		} else {
			this.transported = false; this.transporterUUID = "";
		}
		
	}
}
