/*
** Copyright (C) 2009 Patrick Kirchhof, Kevin Suske and Pasqual Dhring
** 
** This file is part of GFA.
** 
** GFA is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** GFA is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with GFA.  If not, see <http://www.gnu.org/licenses/>.
** 
*/

package animator.src;

import java.awt.Color;

import java.io.File;

public class AnimatorUtils {

	// duration of the animation in seconds
	public static int animationLength = 60;
	// show baseplate in generated x3d file
	public static boolean showBaseplate = true;
	// show grid in generated x3d file
	public static boolean showGrid = true;
	// enable labels
	public static boolean enableLabels = true;
	// enable transporters
	public static boolean enableTransporters = true;
	// use extensive menu / use simple menu
	public static boolean useExtensiveMenu = true;
	// x3d file name
	public static String x3dFileName = "";
	// folders
	public static String menuFolderRel = ".\\menu\\";
	public static String templateFolderRel = ".\\templates\\";
	// folders (development)
	public static String menuFolder = System.getProperty("user.dir")+"\\genericFactoryAnimator3D\\menu\\";
	public static String templateFolder = System.getProperty("user.dir")+"\\genericFactoryAnimator3D\\templates\\";
	// documents
	public static String defaultStructureFile = "Job Shop Experiment_struct.xml";
	public static String defaultTraceFile = "Job Shop Experiment_trace.xml";
	// prefixes
	public static String conveyorPrefix = "CR_";
	public static String itemPrefix = "IM_";
	public static String jobPrefix = "JB_";
	public static String parkingPrefix = "PG_";
	public static String pathPrefix = "PH_";
	public static String queuePrefix = "QE_";
	public static String sinkPrefix = "SI_";
	public static String sourcePrefix = "SO_";
	public static String transporterPrefix = "TP_";
	public static String workstationPrefix = "WS_";
	// current parse mode
	public static int parseMode = -1;
	// parse modes
	public static final int NONE = -1;
	public static final int XML = 0;
	public static final int X3D_STRUCT = 1;
	public static final int X3D_TRACE = 2;
	public static final int OPTIONS = 3;

	/**
	 * Calculate angle between point A(x1,y1) and point B(x2,y2)
	 */
	public static double getAngle(double x1, double x2, double y1, double y2) {
		double tmp1 = y2-y1;
		double tmp2 = x2-x1;
		return Math.atan2(tmp1, tmp2) * 180 / Math.PI; 
	}
	
	/**
	 * Calculate center between two numbers
	 */
	public static double getCenter(double p1, double p2) {
		return (((p2-p1)/2)+p1);
	}
	
	/**
	 * Calculate distance between point A(x1,y1) and point B(x2,y2)
	 */
	public static double getDistance(double x1, double x2, double y1, double y2) {
		double tmp1 = Math.pow((x1-x2), 2);
		double tmp2 = Math.pow((y1-y2), 2);
		return Math.sqrt(tmp1+tmp2);
	}
	
	/**
	 * Returns the name of the prototype 
	 */
	public static String getProtoName(String pathurl) {
		String filename = new File(pathurl).getName();
		int startAt = filename.indexOf("_")+1; int endAt = filename.indexOf(".");
		if ((startAt != -1) && (endAt != -1)) return filename.substring(startAt, endAt);
		return "";
	}
	
	/**
	 * Gets the color from a string
	 * The string is expected to be in the format "r g b"
	 */
	public static Color getRGBColorFromString(String rgb) {
		if (rgb == null) return null;
		float r = Float.valueOf(rgb.substring(0, rgb.indexOf(" ")));
		rgb = rgb.substring(rgb.indexOf(" ")+1, rgb.length());
		float g = Float.valueOf(rgb.substring(0, rgb.indexOf(" ")));
		float b = Float.valueOf(rgb.substring(rgb.indexOf(" ")+1, rgb.length()));
		return new Color(r/255, g/255, b/255);
	}
	
	/**
	 * Calculate time value for x3d from simulation time value
	 */
	public static String getTimeKey(String simulationTime, String simulationLength) {
		double timeKey = Double.parseDouble(simulationTime) / Double.parseDouble(simulationLength);
		return Double.toString(timeKey);
	}
	
	/**
	 * Get x from position string
	 */
	public static String getX(String position) {
		return position.substring(0, position.indexOf(" "));
	}
	
	/**
	 * Get y from position string
	 */
	public static String getY(String position) {
		position = position.substring(position.indexOf(" ")+1, position.length());
		return position.substring(position.indexOf(" ")+1, position.length());
	}
	
	/**
	 * Calculate z-coordinate so that all objects are located on the same level (3d)
	 */
	public static String getZ(String z, double objectHeight) {
		return Double.toString(Double.parseDouble(z)+(objectHeight/2));
	}
	
	/**
	 * Check if a String is a number
	 */
	public static boolean isNumber(String value) {
		for (int i = 0; i < value.toCharArray().length; i++) {
			char c = value.toCharArray()[i];
			if ((!Character.isDigit(c)) && (!(c == '.'))) {
				return false;
			}
		}
	    return true;
	}
	
	/**
	 * True if the selected file contains a prototype (file needs to begin with "Proto_")
	 */
	public static boolean isProtoDeclaration(String pathurl) {
		String filename = new File(pathurl).getName();
		if (filename.length() > 5)
			if (filename.substring(0, 6).toLowerCase().equals("proto_")) return true;
		return false;
	}
}
