/*
** Copyright (C) 2009 Patrick Kirchhof, Kevin Suske and Pasqual Dhring
** 
** This file is part of GFA.
** 
** GFA is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** GFA is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with GFA.  If not, see <http://www.gnu.org/licenses/>.
** 
*/

package animator.src;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;

public class AnimatorXML {
	
	private Document structDoc, traceDoc;
	// used to keep track of all defined path nodes
	private HashMap<String, Vector> pathNodes = new HashMap<String, Vector>();
	
	/**
	 * This class offers methods which create xml nodes for simulation structures and traces
	 */
	public AnimatorXML(boolean initStruct, boolean initTrace) {
		AnimatorUtils.parseMode = AnimatorUtils.XML;
		if (initStruct) structDoc = AnimatorXML.createXMLDocument();
		if (initTrace) traceDoc = AnimatorXML.createXMLDocument();
	}
	
	public void addNode(String uuid, String type, double posx, double posy) {
		Vector<String> v = new Vector<String>(); v.add(0, type); 
		v.add(1, Double.toString(posx)); v.add(2, Double.toString(posy)); v.add(3, "0"); 
		pathNodes.put(uuid, v);
	}
	
	public Element createNodeChangeState(String uuid, double time, String status) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "ChangeStateEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "changeState"));
		e.appendChild(createXMLElement(traceDoc, "state", status));
		return e;
	}
	
	public Element createNodeConveyor(String uuid, String name,
			String originUUID, String destinationUUID, double stdtraveltime,
			double posxfrom, double posyfrom, double posxto, double posyto) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "ComveyorObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "conveyor"));
		e.appendChild(createXMLElement(structDoc, "stdtraveltime", Double.toString(stdtraveltime)));

		if (pathNodes.size()>0) {
			Iterator it = pathNodes.entrySet().iterator();
			while (it.hasNext()) {
		    	Map.Entry mapEntry = (Map.Entry) it.next();	  	
				if (mapEntry.getKey().equals(originUUID)) {
					e.appendChild(createNodeCoordinates(structDoc, "from", Double.parseDouble((String)((Vector)mapEntry.getValue()).get(1)), 
							Double.parseDouble((String)((Vector)mapEntry.getValue()).get(2)), Double.parseDouble((String)((Vector)mapEntry.getValue()).get(3))));
				}
				if (mapEntry.getKey().equals(destinationUUID)) {
					e.appendChild(createNodeCoordinates(structDoc, "to", Double.parseDouble((String)((Vector)mapEntry.getValue()).get(1)), 
							Double.parseDouble((String)((Vector)mapEntry.getValue()).get(2)), Double.parseDouble((String)((Vector)mapEntry.getValue()).get(3))));
				}	
			}
		} else {
			e.appendChild(createNodeCoordinates(structDoc, "from", posxfrom, posyfrom, 0));
			e.appendChild(createNodeCoordinates(structDoc, "to", posxto, posyto, 0));
		}
		
		return e;
	}
	
	public Element createNodeCoordinates(Document doc, String elementName, double x, double y, double z) {
		Element e = doc.createElement(elementName);
		e.appendChild(createXMLElement(doc, "x", Double.toString(x)));
		e.appendChild(createXMLElement(doc, "y", Double.toString(y)));
		e.appendChild(createXMLElement(doc, "z", Double.toString(z)));
		return e;
	}
	
	public Element createNodeCreate(String uuid, double time, String newStatus, double posx, double posy) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "CreateEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "create"));
		
		Iterator it = pathNodes.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
	    	Vector v = ((Vector)entry.getValue());	  	
			if (v.get(0).equals("source") && (posx == Double.parseDouble((String)v.get(1))) && 
					(posy == Double.parseDouble((String)v.get(2)))) {
				e.appendChild(createXMLElement(traceDoc, "source", (String)entry.getKey()));
			}
		}
		
		return e;
	}
	
	public Element createNodeInsert(String uuid, String queueReferenceUUID,
			double time) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "InsertEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "insert"));
		e.appendChild(createXMLElement(traceDoc, "queue", queueReferenceUUID));
		return e;
	}
	
	public Element createNodeItem(String uuid, String name, String template, String color) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "ItemObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "item"));
		e.appendChild(createXMLElement(structDoc, "template", template));
		e.appendChild(createXMLElement(structDoc, "color", color));
		return e;
	}
	
	public Element createNodeJob(String uuid, String customerReferenceUUID,
			double arrivaltime, String itemUuid, double posx, double posy) {	
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "AddJobEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(arrivaltime)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "addJob"));
		e.appendChild(createXMLElement(traceDoc, "item", itemUuid));
		return e;
	}
	
	public Element createNodeLoadTransporter(String uuid, String transporterUuid, double time,
			double posX, double posY) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "LoadTransporterEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "loadTransporter"));
		e.appendChild(createXMLElement(traceDoc, "transporter", transporterUuid));
		return e;
	}
	
	public Element createNodeMoveJob(String uuid, double time, double posX,
			double posY) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "MoveJobEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "moveJob"));
		e.appendChild(createNodeCoordinates(traceDoc, "position", posX, posY, 0));
		return e;
	}
	
	public Element createNodeMoveTransporter(String uuid, double time, double posx,
			double posy) {		
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "MoveTransporterEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "moveTransporter"));
		e.appendChild(createNodeCoordinates(traceDoc, "position", posx, posy, 0));
		return e;
	}	
	
	public Element createNodeParking(String uuid, String name, String type, double posx,
			double posy) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "ParkingObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "parking"));
		e.appendChild(createNodeCoordinates(structDoc, "position", posx, posy, 0));
		return e;
	}
	
	public Element createNodePath(String uuid, String name,
			String originUUID, String destinationUUID, double stdtraveltime,
			double posxfrom, double posyfrom, double posxto, double posyto) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "PathObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "path"));
		e.appendChild(createXMLElement(structDoc, "stdtraveltime", Double.toString(stdtraveltime)));
		
		if (pathNodes.size()>0) {
			Iterator it = pathNodes.entrySet().iterator();
			while (it.hasNext()) {
		    	Map.Entry mapEntry = (Map.Entry) it.next();	  	
				if (mapEntry.getKey().equals(originUUID)) {
					e.appendChild(createNodeCoordinates(structDoc, "from", Double.parseDouble((String)((Vector)mapEntry.getValue()).get(1)), 
							Double.parseDouble((String)((Vector)mapEntry.getValue()).get(2)), Double.parseDouble((String)((Vector)mapEntry.getValue()).get(3))));
				}
				if (mapEntry.getKey().equals(destinationUUID)) {
					e.appendChild(createNodeCoordinates(structDoc, "to", Double.parseDouble((String)((Vector)mapEntry.getValue()).get(1)), 
							Double.parseDouble((String)((Vector)mapEntry.getValue()).get(2)), Double.parseDouble((String)((Vector)mapEntry.getValue()).get(3))));
				}	
			}
		} else {
			e.appendChild(createNodeCoordinates(structDoc, "from", posxfrom, posyfrom, 0));
			e.appendChild(createNodeCoordinates(structDoc, "to", posxto, posyto, 0));
		}
			
		return e;
	}
	
	public Element createNodeQueue(String uuid, String name, 
			double posxIn, double posyIn, double posxOut, double posyOut,
			String queueinUUID, String queueoutUUID) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "QueueObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "queue"));	
		e.appendChild(createNodeCoordinates(structDoc, "entry", posxIn, posyIn, 0));
		e.appendChild(createNodeCoordinates(structDoc, "exit", posxOut, posyOut, 0));
		return e;
	}
	
	public Element createNodeRemove(String uuid, String queueReferenceUUID, String resourceReferenceUUID,
			double crtTime, double serviceTime) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "RemoveEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(crtTime)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "remove"));
		e.appendChild(createXMLElement(traceDoc, "queue", queueReferenceUUID));
		return e;
	}
	
	public Element createNodeSeize(String uuid, String queueReferenceUUID, String resourceReferenceUUID,
			double crtTime, double serviceTime) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "SeizeEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(crtTime)));
		e.appendChild(createXMLElement(traceDoc, "object", resourceReferenceUUID));
		e.appendChild(createXMLElement(traceDoc, "action", "seize"));
		e.appendChild(createXMLElement(traceDoc, "job", uuid));
		e.appendChild(createXMLElement(traceDoc, "duration", Double.toString(serviceTime)));
		return e;
	}
	
	public Element createNodeSink(String uuid, String name, String type, double posx,
			double posy, String template, String rotation) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "SinkObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "sink"));
		e.appendChild(createXMLElement(structDoc, "template", template));
		e.appendChild(createXMLElement(structDoc, "rotation", rotation));
		e.appendChild(createNodeCoordinates(structDoc, "position", posx, posy, 0));
		return e;
	}
	
	public Element createNodeSource(String uuid, String name, String type, double posx,
			double posy, String template, String rotation) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "SourceObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", "source"));
		e.appendChild(createXMLElement(structDoc, "template", template));
		e.appendChild(createXMLElement(structDoc, "rotation", rotation));
		e.appendChild(createNodeCoordinates(structDoc, "position", posx, posy, 0));
		return e;
	}
	
	public Element createNodeStructure() {
		Element e = structDoc.createElement("structure");
		e.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		return e;
	}
	
	public Element createNodeTerminate(String uuid, double time, String newStatus, double posx, double posy) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "TerminateEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "terminate"));

		Iterator it = pathNodes.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
	    	Vector v = ((Vector)entry.getValue());	  	
			if (v.get(0).equals("sink") && (posx == Double.parseDouble((String)v.get(1))) && 
					(posy == Double.parseDouble((String)v.get(2)))) {
				e.appendChild(createXMLElement(traceDoc, "sink", (String)entry.getKey()));
			}
		}
		
		return e;
	}
	
	public Element createNodeTrace() {
		Element e = traceDoc.createElement("trace");
		e.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		return e;
	}
	
	public Element createNodeTransporter(String uuid, String name, 
			double posx, double posy, String type, String template) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "TransporterObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", type));
		e.appendChild(createXMLElement(structDoc, "template", template));		
		e.appendChild(createNodeCoordinates(structDoc, "position", posx, posy, 0));
		return e;
	}
	
	public Element createNodeUnloadTransporter(String uuid, String transporterUuid, double time,
			double posX, double posY) {
		Element e = traceDoc.createElement("event");
		e.setAttribute("xsi:type", "UnloadTransporterEvent");
		e.appendChild(createXMLElement(traceDoc, "time", Double.toString(time)));
		e.appendChild(createXMLElement(traceDoc, "object", uuid));
		e.appendChild(createXMLElement(traceDoc, "action", "unloadTransporter"));
		e.appendChild(createXMLElement(traceDoc, "transporter", transporterUuid));
		return e;
	}
	
	public Element createNodeWorkstation(String uuid, String name, 
			double posx, double posy, String type, String template, String rotation) {
		Element e = structDoc.createElement("object");
		e.setAttribute("xsi:type", "WorkstationObject");
		e.appendChild(createXMLElement(structDoc, "uuid", uuid));
		e.appendChild(createXMLElement(structDoc, "name", name));
		e.appendChild(createXMLElement(structDoc, "type", type));
		e.appendChild(createXMLElement(structDoc, "template", template));
		e.appendChild(createXMLElement(structDoc, "rotation", rotation));
		e.appendChild(createNodeCoordinates(structDoc, "position", posx, posy, 0));
		return e;
	}
	
	/**
	 * Used to create a XML/X3D Document object
	 */
	public static Document createXMLDocument() {
		try {
			DocumentBuilderFactory factory = (DocumentBuilderFactory) DocumentBuilderFactory.newInstance();
			DocumentBuilder b = (DocumentBuilder) factory.newDocumentBuilder();
			return b.newDocument();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Used to create a XML/X3D element object
	 */
	public Element createXMLElement(Document doc, String name, String value) {
		Element element = doc.createElement(name);
		element.appendChild(doc.createTextNode(value));
		return element;
	}
	
	/**
	 * Used to create the physical XML/X3D file
	 */
	public static void createXMLFile(Document doc, String name, String type, String ext, boolean adjustFilename) {
		try {
			if (adjustFilename) {
				if (type != "") name+="_"+type+"."+ext; else name+="."+ext;
			}			
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

			StreamResult result = new StreamResult(new File(name));
			DOMSource source = new DOMSource(doc);
			transformer.transform(source, result);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void finishDocument(String xmlName, Element struct, Element trace, boolean adjustFilename) {
		if (struct != null) {
			structDoc.appendChild(struct);
			createXMLFile(structDoc, xmlName, "struct", "xml", adjustFilename);
		}
		if (trace != null) {
			traceDoc.appendChild(trace);
			createXMLFile(traceDoc, xmlName, "trace", "xml", adjustFilename);
		}
	}
	
	/**
	 * Parse structure and trace XML files using SAX
	 */
	public static void parseXMLFile(String structPath, String tracePath) {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            File structFile = new File(structPath);
            File traceFile = new File(tracePath);
            String fileName =  new File(structPath).getName();
        	if (fileName.indexOf("_") == -1) AnimatorUtils.x3dFileName = fileName.substring(0, fileName.indexOf("."));
        		else AnimatorUtils.x3dFileName = fileName.substring(0, fileName.indexOf("_"));
            AnimatorHandler ahandler = new AnimatorHandler();    
        	
            if (AnimatorUtils.parseMode == AnimatorUtils.OPTIONS) {
            	parser.parse(structFile.toURI().toString(), ahandler);
            } else {
            	AnimatorUtils.parseMode = AnimatorUtils.X3D_STRUCT;
                parser.parse(structFile.toURI().toString(), ahandler);
                AnimatorUtils.parseMode = AnimatorUtils.X3D_TRACE;
                parser.parse(traceFile.toURI().toString(), ahandler);
            }           
        } catch (Exception ex) {
            ex.printStackTrace(System.out);
        }
	}
	
}
